#include "nmod_poly_eta_qexp.h"

static void
_eta_one(
    mp_ptr c,
    slong N,
    nmod_t mod
    )
{
    slong k, n0, n1;

    mp_limb_t m1 = mod.n-1;

    _nmod_vec_zero(c, N);

    /* P */
    for (n0 = 0, n1 = 1, k = 0;
         n1 < N;
         n0 += 6 * k + 5, n1 += 6 * k + 11, k += 2
         )
    {
        c[n0] = 1;
        c[n1] = m1;
    }
    if (n0 < N)
    {
        c[n0] = 1;
    }

    /* Q */
    for (n0 = 2, n1 = 9, k = 1;
        n1 < N;
        n0 += 6 * k + 7, n1 += 6 * k + 13, k += 2
        )
    {
        c[n0] = m1;
        c[n1] = 1;
    }
    if (n0 < N)
    {
        c[n0] = m1;
    }
}

void
_nmod_poly_eta_qexp(
    mp_ptr f,
    slong e,
    slong len,
    nmod_t mod
    )
{
    if (e < 0)
    {
        mp_ptr t = _nmod_vec_init(len);
        _nmod_poly_eta_qexp(t, -e, len, mod);
        _nmod_poly_inv_series(f, t, len, len, mod);
        _nmod_vec_clear(t);
        return;
    }
    else if (e == 0)
    {
        _nmod_vec_zero(f, len);
        if (len > 0)
            f[0] = 1;
    }
    else if (e == 1)
    {
        _eta_one(f, len, mod);
    }
    else
    {
        mp_ptr t;

        t = _nmod_vec_init(len);

        _eta_one(t, len, mod);
        _nmod_poly_pow_trunc(f, t, e, len, mod);

        _nmod_vec_clear(t);
    }
}

void
nmod_poly_eta_qexp(
    nmod_poly_t f,
    slong e,
    slong n
    )
{
    if (n < 1)
    {
        nmod_poly_zero(f);
    }
    else if (e == 0 || n == 1)
    {
        nmod_poly_one(f);
    }
    else
    {
        nmod_poly_fit_length(f, n);
        _nmod_poly_eta_qexp(f->coeffs, e, n, f->mod);
        _nmod_poly_set_length(f, n);
        _nmod_poly_normalise(f);
    }
}
