# completed for lmax = 1000, qmax = 10^11
# completed for lmax = 10000, qmax = 10^9

# exceptions in second run
#  Dict{fmpz,Tuple{Array{fmpz,1},Array{fmpz,1}}} with 2 entries:
#    2351 => (fmpz[], fmpz[455614561])
#    11   => (fmpz[], fmpz[567255721])

using Distributed
@everywhere begin
  using Nemo, Hecke
  include("Primes.jl")
  include("Sieve.jl")
  # maximal prime l to be checked
  lmax = ZZ(10)^4
  # minimal prime q to be checked
  q0 = ZZ(10)^0
  # maximal prime q to be checked
  qmax = ZZ(10)^9
  # maximal obstruction integer
  nmax = ZZ(100)
end

d = @distributed ((d1,d2) -> merge!(vcat,d1,d2)) for l in collect(PrimesSet(ZZ(5),lmax))
  # The case of \delta = 0.
  βl = mod(divexact(1-l^2, 24), l);
  # List of obstruction integers
  int_obstr = [divexact(24*n-1,l) for n in βl:l:l*nmax 
               if !divisible(number_of_partitions(n),l)];
  if l ∉ fmpz[5,7,11]
    delta0 = filter!(q -> q ∉ fmpz[2,3,5,7,11,l],
                     sieve(int_obstr, qmax, q0))
  else
    delta0 = fmpz[]
  end
  # The case of \delta = -1
  int_obstr = [24*n-1 for n in 1:nmax 
               if jacobi_symbol(1-24*n,l) == -1 &&
                  !divisible(number_of_partitions(n),l)];
  deltaneg = filter!(q -> q ∉ fmpz[2,3,5,7,11,l],
                     sieve(int_obstr, qmax, q0))
  isempty(delta0) && isempty(deltaneg) ?
    Dict{fmpz,Tuple{Vector{fmpz},Vector{fmpz}}}() :
    Dict(l => (delta0,deltaneg))
end
