using Nemo, Hecke

function kloosterman(Q::Int)
  for prec in Iterators.countfrom(1000,1000)
    @info "" prec
    CC = ComplexField(prec)
    roots = [exppii(CC(2*ix*jx//Q)) for ix in 1:Q-1, jx in 1:Q]

    sums = [zero(CC) for ix in 1:Q-1, a in 1:Q-1]
    
    F = GF(Q)
    for x in 1:Q-1
      x = F(x)
      xinv = inv(x)
      for a in 1:Q-1
        jx = Int(lift(x + a*xinv))
        if jx == 0
          jx = Q
        end
        for ix in 1:Q-1
          sums[ix,a] = addeq!(sums[ix,a], roots[ix,jx])
        end
      end
    end
  
    return_flag = true
    norms = Vector{fmpz}(undef,Q-1)
    for a in 1:Q-1
      flag,norm = unique_integer(real(prod([sums[ix,a] for ix in 1:Q-1])))
      if !flag
        return_flag = false
        break
      end
      norms[a] = norm
    end
    return_flag && return norms
  end
end

ell = 11
for Q in Hecke.PrimesSet(239,1000)
  @info "" Q
  Q == ell && continue
  kl2 = kloosterman(Q)
  if any(divisible(v,ell) for v in kl2)
    println("ell=$ell Q=$Q $kl2")
    break
  end
end


using Profile, ProfileView

@profile kloosterman(139);
ProfileView.view()


kl = kloosterman(139);
Nemo.factor(kl[70])
